-- vim: ts=4 sw=4 ai et

-- We need to have the localization table set up before this file is loaded.
local L = Ammo.strings


local ammotable = {
    premium = L["Premium"],
    lowgrade = L["Low-grade"]
}

local function mkchatoption (ord, disf)
    local t = {
        type = 'toggle',
        name = L["Chat"],
        desc = L["Provide warning in the default chat window."],
        order = ord,
        disabled = disf,
    }
    return t
end

local function mkwarnoption (ord, disf)
    local t = {
        type = 'toggle',
        name = L["Error frame"],
        desc = L["Provide warning in the standard error frame (top center of screen)."],
        order = ord,
        disabled = disf,
    }
    return t
end

local function mkvoiceoption (ord, disf)
    local t = {
        type = 'toggle',
        name = L["Voice"],
        desc = L["Provide vocal warning."],
        disabled = disf,
        order = ord,
    }
    return t
end

local function mkautoswitchoption (ord)
    local t = {
        type = 'toggle',
        name = L["Auto-switch enabled"],
        desc = L["Automatically switch ammo when in this environment. Only available to configure if switching is enabled on the main settings panel."],
        disabled = function () return not Ammo.db.profile.general_switch end,
        order = ord,
    }
    return t
end

local function mkbossesammoselect (ord, disf)
    local t = {
        type = 'select',
        name = L["Bosses"],
        desc = L["Ammo type to use when fighting bosses."],
        values = ammotable,
        order = ord,
        disabled = disf,
    }
    return t
end

local function mktrashammoselect (ord, disf)
    local t = {
        type = 'select',
        name = L["Trash"],
        desc = L["Ammo type to use when fighting trash mobs."],
        values = ammotable,
        order = ord,
        disabled = disf
    }
    return t
end

local function fcl ()
    return not Ammo.db.profile.current_low_enabled
end

local function fce ()
    return not Ammo.db.profile.current_exhausted_enabled
end

local function fae ()
    return not Ammo.db.profile.all_exhausted_enabled
end

local function wasm ()
    return not Ammo.db.profile.wrong_enabled
end

local function glue_ammo_types_available ()
    local links = Ammo.addon:AvailableAmmoTypes ()
    local table = {}
    for _, link in ipairs (links) do
        table[link] = link
    end
    -- Ensure we list the currently selected ammo type, even if we don't have any of it
    local ptype = Ammo.db.profile.restock_premium_type
    local ltype = Ammo.db.profile.restock_lowgrade_type
    if ptype and not table[ptype] then table[ptype] = ptype end
    if ltype and not table[ltype] then table[ltype] = ltype end
    return table
end

local function bag_list ()
    local t = {}
    for i = 0, 4 do
        local n = GetBagName (i)
        if n then t[i] = (i+1).." "..n end
    end
    return t
end

Ammo.cfghandler = {}

Ammo.config = {
    type = 'group',
    handler = Ammo.cfghandler,
    get = "GetVal", set = "SetVal",
    args = {
        general = {
            type = 'group',
            name = L["General settings"],
            args = {
                blurb = {
                    type = 'description',
                    order = 1,
                    name = L["Ammo-matique provides automated out-of-combat ammo handling. It can automatically select the suitable ammo type in any particular situation."],
                },
                blurb2 = {
                    type = 'description',
                    order = 2,
                    name = L["Ammo-matique uses two terms when referring to ammo - 'premium' and 'lowgrade'. Premium refers to the ammo type with the highest DPS that you have available. Conversely, lowgrade refers to the ammo type with the lowest DPS."],
                },
                blurb3 = {
                    type = 'description',
                    order = 3,
                    name = L["Tip: you can use /ammo to access Ammo-matique functionality from macros or the chat box."],
                },
                hdr = {
                    type = 'header',
                    name = "",
                    order = 19,
                },
                switch = {
                    type = 'toggle',
                    name = L["Auto-switch"],
                    desc = L["Automatically switch ammo in PvE and/or PvP."],
                    order = 30,
                },
                notify = {
                    type = 'toggle',
                    name = L["Notifications"],
                    desc = L["Provide status notifications, e.g. when switching ammo."],
                    order = 40,
                },
                spacer = {
                    type = 'description',
                    order = 99,
                    name = "\n\n\n\n\n\n\n",
                },
                hdr2 = {
                    type = 'header',
                    name = "",
                    order = 100,
                },
                author = {
                    type = 'description',
                    name = "By Anyia of Horde Yakka (Jubei'Thos)\n",
                    order = 101,
                },
                thanks = {
                    type = 'description',
                    name = "With special thanks to:\n  Gagorian - for a clever solution to a localization issue\n",
                    order = 102,
                },
            }
        },

        warnings = {
            type = 'group',
            name = L["Ammo warnings"],
            args = {
                blurb = {
                    type = 'description',
                    order = 1,
                    name = L["Ammo-matique can provide warnings when you are running low or out of ammunition in a variety of ways. Select the ones you would like."],
                },
                voiceset = {
                    type = 'select',
                    order = 5,
                    name = L["Voice set"],
                    desc = L["Voice set to use if voice warnings are enabled."],
                    values = { male1 = L["Male"], female1 = L["Female"] },
                },
                current_low = {
                    type = 'group',
                    inline = true,
                    name = L["Low ammo warning"],
                    order = 10,
                    args = {
                        enabled = {
                            type = 'toggle',
                            name = L["Enabled"],
                            desc = L["Warn when the current ammo is running low."],
                            order = 10,
                        },
                        threshold = {
                            type = 'range',
                            name = L["Low ammo threshold"],
                            desc = L["Warn when current ammo falls below this value."],
                            order = 20,
                            min = 0, max = 2000, step = 20,
                            disabled = fcl,
                        },
                        ignoreme = {
                            type = 'description',
                            name = "",
                            order = 29,
                        },
                        text = mkchatoption (30, fcl),
                        warn = mkwarnoption (35, fcl),
                        voice = mkvoiceoption (40, fcl),
                    },
                }, -- End: Low ammo warning
                current_exhausted = {
                    type = 'group',
                    inline = true,
                    name = L["Ammo exhausted warning"],
                    order = 20,
                    args = {
                        enabled = {
                            type = 'toggle',
                            name = L["Enabled"],
                            desc = L["Warn when the current ammo is exhausted."],
                            order = 10,
                        },
                        ignoreme = {
                            type = 'description',
                            name = "",
                            order = 29,
                        },
                        text = mkchatoption (30, fce),
                        warn = mkwarnoption (35, fce),
                        voice = mkvoiceoption (40, fce),
                    },
                }, -- End current_exhausted
                all_exhausted = {
                    type = 'group',
                    inline = true,
                    name = L["All ammo exhausted warning"],
                    order = 30,
                    args = {
                        enabled = {
                            type = 'toggle',
                            name = L["Enabled"],
                            desc = L["Warn when all ammo is exhausted."],
                            order = 10,
                        },
                        ignoreme = {
                            type = 'description',
                            name = "",
                            order = 29,
                        },
                        text = mkchatoption (30, fae),
                        warn = mkwarnoption (35, fae),
                        voice = mkvoiceoption (40, fae),
                    },
                }, -- End all_exhausted
                wrong = {
                    type = 'group',
                    inline = true,
                    name = L["Wrong ammo used warning"],
                    order = 40,
                    args = {
                        enabled = {
                            type = 'toggle',
                            name = L["Enabled"],
                            desc = L["Warn if using the wrong ammo when in combat, and Ammo-matique can not automatically switch it for you."],
                            order = 10,
                        },
                        ignoreme = {
                            type = 'description',
                            name = "",
                            order = 29,
                        },
                        text = mkchatoption (30, wasm),
                        warn = mkwarnoption (35, wasm),
                        voice = mkvoiceoption (40, wasm),
                    },
                }, -- End wrong ammo used
            }
        },

        pve = {
            type = 'group',
            name = L["PvE ammo switching"],
            childGroups = 'tab',
            args = {
                blurb = {
                    type = 'description',
                    order = 1,
                    name = L["Ammo-matique allows you to automatically switch ammo depending on the type of mob you are about to fight."],
                },

                world = {
                    type = 'group',
                    name = L["World"],
                    order = 10,
                    get = "GetWorldConfig", -- Ammo.cfghandler:GetWorldConfig
                    set = "SetWorldConfig",
                    args = {
                        blurb = { type = 'description', order = 1, name = L["Mobs roaming the world outside any instances."] },
                        header = { type = 'header', order = 2, name = "" },
                        enabled = mkautoswitchoption (10),
                        spacer = { type = 'description', name = " ", order = 11 },
                        worldboss = mkbossesammoselect (20, function() return not Ammo.db.profile.general_switch or not Ammo.db.profile.world_settings.enabled end),
                        elite = {
                            type = 'select',
                            name = L["Elites"],
                            desc = L["Ammo type to use when fighting elite mobs."],
                            order = 30,
                            disabled = function() return not Ammo.db.profile.general_switch or not Ammo.db.profile.world_settings.enabled end,
                            values = ammotable,
                        },
                        normal = {
                            type = 'select',
                            name = L["Non-elites"],
                            desc = L["Ammo type to use when fighting non-elite mobs."],
                            order = 40,
                            disabled = function() return not Ammo.db.profile.general_switch or not Ammo.db.profile.world_settings.enabled end,
                            values = ammotable,
                        },
                    }
                },

                dungeons = {
                    type = 'group',
                    name = L["Dungeons"],
                    order = 15,
                    get = "GetDungeonConfig", -- Ammo.cfghandler:GetDungeonConfig
                    set = "SetDungeonConfig",
                    args = {
                        blurb = { type = 'description', order = 1, name = L["Mobs inside 5-player dungeons."], },
                        header_normal = {
                            type = 'header',
                            order = 10,
                            name = L["Normal dungeons"],
                        },
                        enabled = mkautoswitchoption (20),
                        spacer = { type = 'description', name = " ", order = 21 },
                        worldboss = mkbossesammoselect (30, function() return not Ammo.db.profile.general_switch or not Ammo.db.profile.dungeon_settings.enabled end),
                        trash  = mktrashammoselect  (40, function() return not Ammo.db.profile.general_switch or not Ammo.db.profile.dungeon_settings.enabled end),
                        header_heroic = {
                            type = 'header',
                            order = 110,
                            name = L["Heroic dungeons"],
                        },
                        heroic_enabled = mkautoswitchoption (120),
                        heroic_spacer = { type = 'description', name = " ", order = 121 },
                        heroic_worldboss = mkbossesammoselect (130, function() return not Ammo.db.profile.general_switch or not Ammo.db.profile.dungeon_settings.heroic_enabled end),
                        heroic_trash  = mktrashammoselect  (140, function() return not Ammo.db.profile.general_switch or not Ammo.db.profile.dungeon_settings.heroic_enabled end),
                    }
                },

                raids = {
                    type = 'group',
                    name = L["Raids"],
                    order = 19,
                    get = "GetRaidConfig", -- Ammo.cfghandler:GetRaidConfig
                    set = "SetRaidConfig",
                    -- args are handled via metatable below
                },
            },
        },

        pvp = {
            type = 'group',
            name = L["PvP ammo switching"],
            disabled = function () return not Ammo.db.profile.general_switch end,
            args = {
                blurb = {
                    type = 'description',
                    order = 1,
                    name = L["Whenever you engage (or prepare to engage) another player in combat, Ammo-matique can automatically switch in ammo for you based on the situation."],
                },
                arena = {
                    type = 'group',
                    inline = true,
                    name = L["Arena"],
                    order = 10,
                    args = {
                        enabled = {
                            type = 'toggle',
                            name = L["Enabled"],
                            desc = L["Switch ammo when engaging in an Arena."],
                            order = 10,
                        },
                        ammotype = {
                            type = 'select',
                            name = L["Ammo type"],
                            desc = L["Ammo type to use when engaging in an Arena."],
                            order = 20,
                            disabled = function () return not Ammo.db.profile.arena_enabled or not Ammo.db.profile.general_switch end,
                            values = ammotable,
                        },
                    },
                },
                bg = {
                    type = 'group',
                    inline = true,
                    name = L["Battlegrounds"],
                    order = 20,
                    args = {
                        enabled = {
                            type = 'toggle',
                            name = L["Enabled"],
                            desc = L["Switch ammo when engaging in Battlegrounds."],
                            order = 10,
                        },
                        ammotype = {
                            type = 'select',
                            name = L["Ammo type"],
                            desc = L["Ammo type to use when engaging in Battlegrounds."],
                            order = 20,
                            disabled = function () return not Ammo.db.profile.bg_enabled or not Ammo.db.profile.general_switch end,
                            values = ammotable,
                        },
                    },
                },
                world = {
                    type = 'group',
                    inline = true,
                    name = L["World PvP"],
                    order = 30,
                    args = {
                        enabled = {
                            type = 'toggle',
                            name = L["Enabled"],
                            desc = L["Switch ammo when engaging in world PvP."],
                            order = 10,
                        },
                        ammotype = {
                            type = 'select',
                            name = L["Ammo type"],
                            desc = L["Ammo type to use when engaging in world PvP."],
                            order = 20,
                            disabled = function () return not Ammo.db.profile.world_enabled or not Ammo.db.profile.general_switch end,
                            values = ammotable,
                        },
                    },
                },
            }
        }, -- end pvp section

        restock = {
            type = 'group',
            name = L["Ammo restocking"],
            args = {
                blurb = {
                    type = 'description',
                    order = 1,
                    name = L["Ammo-matique can automatically restock your ammo supply whenever you visit a vendor selling ammo. If you only have one ammo type in your inventory, it is considered the premium type and will be restocked accordingly unless you have specified set ammo.\nOnly enable this if you are comfortable having an add-on buy things automatically - it could be expensive!"],
                },
                premium = {
                    type = 'toggle',
                    order = 10,
                    name = L["Restock premium"],
                    desc = L["Restock premium ammo automatically when visiting an ammo vendor."]
                },
                restock_premium = {
                    type = 'group',
                    inline = true,
                    name = L["Restock options"],
                    order = 11,
                    disabled = function () return not Ammo.db.profile.restock_premium end,
                    args = {
                        count = {
                            type = 'range',
                            order = 10,
                            min = 0, max = 20000, step = 1, bigStep = 200,
                            name = L["Ammo stock level"],
                            desc = L["Ammo stock level to adhere to (individual arrows/bullets, not stacks)"],
                        },
                        spacer = { type = 'description', order = 11, name = "" },
                        fixed = {
                            type = 'toggle',
                            order = 20,
                            name = L["Set ammo type"],
                            desc = L["Always buy the specified ammo type for premium ammo (as opposed to automatically discovering it)"],
                        },
                        type = {
                            type = 'select',
                            order = 30,
                            disabled = function () return not Ammo.db.profile.restock_premium or not Ammo.db.profile.restock_premium_fixed end,
                            name = L["Ammo type"],
                            desc = L["Always buy this ammo type when restocking. If the desired ammo type is not listed, please purchase a sample of it and come back here."],
                            values = glue_ammo_types_available,
                        },
                    },
                },
                lowgrade = {
                    type = 'toggle',
                    order = 100,
                    name = L["Restock low-grade"],
                    desc = L["Restock low-grade ammo automatically when visiting an ammo vendor."]
                },
                restock_lowgrade = {
                    type = 'group',
                    inline = true,
                    name = L["Restock options"],
                    order = 101,
                    disabled = function () return not Ammo.db.profile.restock_lowgrade end,
                    args = {
                        count = {
                            type = 'range',
                            order = 10,
                            min = 0, max = 20000, step = 1, bigStep = 200,
                            name = L["Ammo stock level"],
                            desc = L["Ammo stock level to adhere to (individual arrows/bullets, not stacks)"],
                        },
                        spacer = { type = 'description', order = 11, name = "" },
                        fixed = {
                            type = 'toggle',
                            order = 20,
                            name = L["Set ammo type"],
                            desc = L["Always buy the specified ammo type for low-grade ammo (as opposed to automatically discovering it)"],
                        },
                        type = {
                            type = 'select',
                            order = 30,
                            disabled = function () return not Ammo.db.profile.restock_lowgrade or not Ammo.db.profile.restock_lowgrade_fixed end,
                            name = L["Ammo type"],
                            desc = L["Always buy this ammo type when restocking. If the desired ammo type is not listed, please purchase a sample of it and come back here."],
                            values = glue_ammo_types_available,
                        },
                    },
                },
            },
        }, -- end restock section

        ammobag = {
            type = 'group',
            order = 60,
            name = L["Ammo bag"],
            args = {
                blurb = {
                    type = 'description',
                    order = 10,
                    name = L["Ammo-matique can ensure all your ammo is kept in a single bag even when you are not using quivers or ammo pouches. Designate which bag to use as the ammo bag below."],
                },
                enabled = {
                    type = 'toggle',
                    order = 20,
                    name = L["Use ammo bag"],
                    desc = L["Automatically keep ammo in the single designated bag whenever possible."],
                },
                bagid = {
                    type = 'select',
                    order = 30,
                    name = L["Ammo bag"],
                    desc = L["Select bag to use as the designated ammo bag"],
                    disabled = function () return not Ammo.db.profile.ammobag_enabled end,
                    values = bag_list,
                }
            }
        }, -- end ammobag section

    }
}


-- Base config panel for raid ammo config MkII, used to build the actual panel via metatable
local RaidAmmoSettings = {
    blurb = {
        type = 'description',
        order = 10,
        name = L["Mobs inside raid instances."].." "..L["Raid instances are added here when you enter them the first time."],
    },
    raids = {
        type = 'header',
        order = 50,
        name = L["Known raid instances"],
    },
}

-- "disabled" handler for the below template
local function RaidAmmoSelectionTemplateDisabledFunction (info)
    info[#info] = "enabled" -- Careful...
    return not Ammo.cfghandler:GetRaidConfig (info) or not Ammo.db.profile.general_switch
end
-- Template for the per-raid-type ammo settings
local RaidAmmoSelectionTemplate = {
    enabled = mkautoswitchoption (10), -- the name of this entry is used by the above function!
    spacer = { type = 'description', name = " ", order = 11 },
    worldboss = mkbossesammoselect (20, RaidAmmoSelectionTemplateDisabledFunction),
    trash = mktrashammoselect (30, RaidAmmoSelectionTemplateDisabledFunction),
}

-- Template for per-raid ammo settings
local RaidAmmoSettingsTemplate = {
    type = 'group',
    name = function(info) return info[#info] end, -- Inherit our name from the key to this entry
    args = {
        blurb = {
            type = 'description',
            name = L["Ammo switching preferences for this raid are available for the different raid modes on the left.\n\nIt is also possible to remove this raid instance and all associated settings."],
            order = 10
        },
        remove = {
            type = 'execute',
            name = L["Remove this raid"],
            desc = L["Remove all the configuration settings for this raid (on this character only)."],
            func = function(info)
                -- Note to self: http://www.wowwiki.com/Creating_simple_pop-up_dialog_boxes
                local raidname = info[#info -1]
                local TAG = "Ammo-matique-REMOVERAID"
                if not StaticPopupDialogs[TAG] then
                    StaticPopupDialogs[TAG] = {
                        text = tostring(Ammo.addon).."\n\n"..L["Are you sure you wish to remove this raid and all associated settings?"],
                        button1 = L["Yes"],
                        button2 = L["No"],
                        OnAccept = function (self, raidname) Ammo.cfg.unregister_raid (raidname) end,
                        timeout = 0,
                        whileDead = true,
                        hideOnEscape = true,
                    }
                end
                local dlg = StaticPopup_Show(TAG)
                if dlg then dlg.data = raidname end
            end,
            order = 20,
        },
        r10 = { type = 'group', name = RAID_DIFFICULTY_10PLAYER, order = 100, args = RaidAmmoSelectionTemplate },
        r10h = { type = 'group', name = RAID_DIFFICULTY_10PLAYER_HEROIC, order = 200, args = RaidAmmoSelectionTemplate },
        r25 = { type = 'group', name = RAID_DIFFICULTY_25PLAYER, order = 300, args = RaidAmmoSelectionTemplate },
        r25h = { type = 'group', name = RAID_DIFFICULTY_25PLAYER_HEROIC, order = 400, args = RaidAmmoSelectionTemplate },
    },
}

-- Meta table for the "raids" args. Dynamically adds entries for each known raid instance
local xt = getmetatable(Ammo.config.args.pve.args.raids) or {}
xt.__index = function (table, key)
  if key == "args" then
    local res = {}
    for k,v in pairs (RaidAmmoSettings) do
        res[k] = v
    end
    for i, n in ipairs (Ammo.db.profile.known_raid_instances) do
        --res[n] = RaidAmmoSettingsTemplate -- If we don't copy, we can't assign the order :(
        res[n] = {}
        local t = res[n]
        for k,v in pairs (RaidAmmoSettingsTemplate) do
            t[k] = v
        end
        t.order = i
    end
    if #Ammo.db.profile.known_raid_instances == 0 then
        res["none"] = { type = 'description', name = L["You have not entered any raid instances yet."], order = 100 }
    end
    return res
  end
end
setmetatable(Ammo.config.args.pve.args.raids, xt)


-- Config handlers

function Ammo.cfghandler:GetVal (info)
    local varname = info[#info -1].."_"..info[#info]
    return Ammo.db.profile[varname]
end

function Ammo.cfghandler:SetVal (info, v)
    local varname = info[#info -1].."_"..info[#info]
    Ammo.db.profile[varname] = v
end

function Ammo.cfghandler:GetRaidConfig (info)
    local raidname = info[#info -2]
    local settings = Ammo.db.profile.raid_settings[raidname] or {}
    local varname = info[#info -1].."_"..info[#info]
    return settings[varname]
end

function Ammo.cfghandler:SetRaidConfig (info, v)
    local raidname = info[#info -2]
    if not Ammo.db.profile.raid_settings[raidname] then
        Ammo.db.profile.raid_settings[raidname] = {}
    end
    local settings = Ammo.db.profile.raid_settings[raidname]
    local varname = info[#info -1].."_"..info[#info]
    settings[varname] = v
end

function Ammo.cfghandler:GetDungeonConfig (info)
    local varname = info[#info]
    return Ammo.db.profile.dungeon_settings[varname]
end

function Ammo.cfghandler:SetDungeonConfig (info, v)
    local varname = info[#info]
    Ammo.db.profile.dungeon_settings[varname] = v
end

function Ammo.cfghandler:GetWorldConfig (info)
    return Ammo.db.profile.world_settings[info[#info]]
end

function Ammo.cfghandler:SetWorldConfig (info, v)
    Ammo.db.profile.world_settings[info[#info]] = v
end

-- Default values
Ammo.defaults = {
    profile = {
        feature_level = 1,

        general_switch = true,
        general_notify = true,

        warnings_voiceset = "female1",
        current_low_enabled = true,
        current_low_threshold = 400,
        current_low_text = true,
        current_low_warn = false,
        current_low_voice = true,
        current_exhausted_enabled = true,
        current_exhausted_text = true,
        current_exhausted_warn = false,
        current_exhausted_voice = true,
        all_exhausted_enabled = true,
        all_exhausted_text = true,
        all_exhausted_warn = false,
        all_exhausted_voice = true,
        wrong_enabled = true,
        wrong_text = true,
        wrong_warn = false,
        wrong_voice = true,

        arena_enabled = true,
        arena_ammotype = "premium",
        bg_enabled = false,
        bg_ammotype = "lowgrade",
        world_enabled = false,
        world_ammotype = "lowgrade",

        -- Restock settings
        restock_premium = false,
        restock_premium_count = 0,
        restock_premium_fixed = false,
        restock_premium_type = nil,
        restock_lowgrade = false,
        restock_lowgrade_count = 1000,
        restock_lowgrade_fixed = false,
        restock_lowgrade_type = nil,

        -- Ammobag settings
        ammobag_enabled = false,
        ammobag_bagid = 0,

        -- MkII settings
        known_raid_instances = {},
        raid_settings = {},
        template_raid_settings = {
            r10_enabled  = false, r10_worldboss  = 'premium', r10_trash  = 'lowgrade',
            r10h_enabled = false, r10h_worldboss = 'premium', r10h_trash = 'lowgrade',
            r25_enabled  = false, r25_worldboss  = 'premium', r25_trash  = 'lowgrade',
            r25h_enabled = false, r25h_worldboss = 'premium', r25h_trash = 'lowgrade',
        },
        dungeon_settings = {
            enabled = false,
            worldboss = 'premium',
            trash = 'lowgrade',
            heroic_enabled = false,
            heroic_worldboss = 'premium',
            heroic_trash = 'lowgrade',
        },
        world_settings = {
            enabled = false,
            worldboss = 'premium',
            elite = 'premium',
            normal = 'lowgrade',
        },
    }
}

--
-- Entry points for Ammo-matique to get to ammo selection data
--
Ammo.cfg = {}
function Ammo.cfg.register_raid (zone_name)
    for i,n in ipairs (Ammo.db.profile.known_raid_instances) do
        if n == zone_name then return end
    end
    tinsert (Ammo.db.profile.known_raid_instances, 1, zone_name)
    Ammo.db.profile.raid_settings[zone_name] = {}
    local t = Ammo.db.profile.raid_settings[zone_name]
    for k,v in pairs (Ammo.defaults.profile.template_raid_settings) do
        t[k] = v
    end
    LibStub:GetLibrary("AceConfigRegistry-3.0"):NotifyChange (Ammo.NAME)

    local TAG = "Ammo-matique-NEWRAID"
    if not StaticPopupDialogs[TAG] then
        StaticPopupDialogs[TAG] = {
            text = tostring(Ammo.addon).."\n\n"..zone_name.."\n\n"..L["You have not yet configured ammo preferences for this raid.\n\nWould you like to do so now?"],
            button1 = L["Yes"],
            button2 = L["No"],
            OnAccept = function (self, raidname)
                local acd = LibStub:GetLibrary ("AceConfigDialog-3.0")
                acd:Open (Ammo.NAME, "pve")
                acd:SelectGroup (Ammo.NAME, "pve", "raids", raidname)
            end,
            timeout = 0,
            whileDead = true,
            hideOnEscape = true,
        }
    end
    local dlg = StaticPopup_Show(TAG)
    if dlg then dlg.data = zone_name end
end

function Ammo.cfg.unregister_raid (raidname)
    for i,n in ipairs (Ammo.db.profile.known_raid_instances) do
        if n == raidname then
            tremove (Ammo.db.profile.known_raid_instances, i)
            break
        end
    end
    Ammo.db.profile.raid_settings[raidname] = nil
    LibStub:GetLibrary("AceConfigRegistry-3.0"):NotifyChange (Ammo.NAME)
end

function Ammo.cfg.ammolookup (instance_type, difficulty, unit_type, zone_name)
    -- instance_type is: none, party, raid
    -- difficulty is:    1..2 for "party", 1..4 for "raid", ignored for "none"
    -- unit_type is:     worldboss, elite, normal
    -- zone_name is:     value from GetZoneText(), ideally cached
    if instance_type == "party" then
        local t = Ammo.db.profile.dungeon_settings
        if difficulty == 1 then
            if not t.enabled then return nil end
            if unit_type ~= "worldboss" then unit_type = "trash" end
            return t[unit_type]
        else
            if not t.heroic_enabled then return nil end
            if unit_type == "worldboss" then unit_type = "heroic_worldboss" else unit_type = "heroic_trash" end
            return t[unit_type]
        end

    elseif instance_type == "raid" then
        local settings = Ammo.db.profile.raid_settings[zone_name]
        if not settings then
          Ammo.cfg.register_raid (zone_name)
          settings = Ammo.db.profile.raid_settings[zone_name]
        end
        local diff_prefixes = { "r10_", "r25_", "r10h_", "r25h_" }
        local prefix = diff_prefixes[difficulty]
        if not settings[prefix.."enabled"] then return nil end
        if unit_type == "worldboss" then
            return settings[prefix.."worldboss"]
        else
            return settings[prefix.."trash"]
        end

    else
        local t = Ammo.db.profile.world_settings
        if not t.enabled then return nil end
        return t[unit_type]
    end

end

